#!/bin/bash

# Abort execution when an error occurs
set -e

# Set working directory properly
cd "$(dirname $0)"
WORKDIR="$(pwd)"

function generateEnvironments {
  if [ -z ${PROD} ]; then
    CURRENT_DOMAIN="localhost"
    LOGGING_LEVEL_ROOT="INFO"
    APP_MAIL_HOST="fragjetzt-mailhog"
  else
    read -p "Domain: " CURRENT_DOMAIN
    LOGGING_LEVEL_ROOT="ERROR"
    APP_MAIL_HOST="fragjetzt-postfix"
  fi

  CURRENT_PORT="80"
  APP_MAIL_SENDER_ADDRESS="postmaster@${CURRENT_DOMAIN}"
  SERVER_ROOT_URL="http:\/\/${CURRENT_DOMAIN}:${CURRENT_PORT}\/"
  MAIL_DOMAIN="${CURRENT_DOMAIN}"
  MAIL_HOST="${CURRENT_DOMAIN}"
  ALLOWEDORIGINS="http:\/\/${CURRENT_DOMAIN}:${CURRENT_PORT}"
}

function generateSecrets {
  if [ -z ${PROD} ]; then
    POSTGRES_SECRET="fragjetzt"
    RABBITMQ_SECRET="guest"
    JWT_SECRET=$(openssl rand --hex 32)
  else
    POSTGRES_SECRET=$(openssl rand --hex 32)
    RABBITMQ_SECRET=$(openssl rand --hex 32)
    JWT_SECRET=$(openssl rand --hex 32)
  fi
}

function createEnvironments {

  if [ ! -f environments/generated.env ] || [ ! -z ${RECREATE_ENV} ]; then

    # CONFIG ENV
    for FILE in environments/*.template; do
      cat "$FILE" | \
        sed "s/^APP_MAIL_HOST=\$/APP_MAIL_HOST=${APP_MAIL_HOST}/" | \
        sed "s/^APP_MAIL_SENDER_ADDRESS=\$/APP_MAIL_SENDER_ADDRESS=${APP_MAIL_SENDER_ADDRESS}/" | \
        sed "s/^SERVER_ROOT_URL=\$/SERVER_ROOT_URL=${SERVER_ROOT_URL}/" | \
        sed "s/^LOGGING_LEVEL_ROOT=\$/LOGGING_LEVEL_ROOT=${LOGGING_LEVEL_ROOT}/" | \
        sed "s/^MAIL_DOMAIN=\$/MAIL_DOMAIN=${MAIL_DOMAIN}/" | \
        sed "s/^MAIL_HOST=\$/MAIL_HOST=${MAIL_HOST}/" | \
        sed "s/^ALLOWEDORIGINS=\$/ALLOWEDORIGINS=${ALLOWEDORIGINS}/" \
      > "${FILE%.template}.env";
    done


    # SECRET ENV
    for FILE in environments/*.template; do
      sed -i "s/^SPRING_RABBITMQ_PASSWORD=\$/SPRING_RABBITMQ_PASSWORD=${RABBITMQ_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^RABBITMQ_DEFAULT_PASS=\$/RABBITMQ_DEFAULT_PASS=${RABBITMQ_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^STOMP_RELAY_PASSWORD=\$/STOMP_RELAY_PASSWORD=${RABBITMQ_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^RABBITMQ_PASSWORD=\$/RABBITMQ_PASSWORD=${RABBITMQ_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^SPRING_R2DBC_PASSWORD=\$/SPRING_R2DBC_PASSWORD=${POSTGRES_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^POSTGRES_PASSWORD=\$/POSTGRES_PASSWORD=${POSTGRES_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^SPRING_JWT_SECRET=\$/SPRING_JWT_SECRET=${JWT_SECRET}/" "${FILE%.template}.env"
      sed -i "s/^SECURITY_JWT_SECRET=\$/SECURITY_JWT_SECRET=${JWT_SECRET}/" "${FILE%.template}.env"
    done

    # BUILD ENV
    ln -fs rabbitmq_plugins.dev configs/rabbitmq_plugins

    date > environments/generated.env

  fi
}

function createComposeOverride {
  cat > docker-compose.override.yml <<MODULES
version: '3.8'

services:
$(cat override-services/rabbitmq.dev)

$([ -z ${NO_BACKEND} ] && cat override-services/backend.dev || cat override-services/backend.disabled)

$([ -z ${NO_WS_GATEWAY} ] && cat override-services/ws-gateway.dev || cat override-services/ws-gateway.disabled)

$([ -z ${NO_FRONTEND} ] && cat override-services/frontend.dev || cat override-services/frontend.disabled)

$([ -z ${NO_EMAIL} ] && cat override-services/email.dev || cat override-services/email.disabled)

volumes:
  fragjetzt-maildir:
    name: fragjetzt-maildir
MODULES
}

while true; do
  case "$1" in
    --prod ) PROD=true; shift ;;
    --no-backend ) NO_BACKEND=true; shift ;;
    --no-frontend ) NO_FRONTEND=true; shift ;;
    --no-ws-gateway ) NO_WS_GATEWAY=true; shift ;;
    --no-email ) NO_EMAIL=true; shift ;;
    --recreate-env ) RECREATE_ENV=true; shift ;;
    "" ) break ;;
    * ) echo "unknown option: $1"; exit 1 ;;

  esac
done

generateEnvironments
generateSecrets
createEnvironments

[ -z $PROD ] && createComposeOverride || rm -f docker-compose.override.yml
